<?php

namespace Drupal\content_insights_report\Controller;

use Drupal\Core\Database\Connection;
use Drupal\Core\Database\Query\PagerSelectExtender;
use Drupal\Core\Datetime\DateFormatterInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\Core\Form\FormBuilderInterface;
use Drupal\Core\Pager\PagerManagerInterface;
use Drupal\Core\Pager\PagerParameters;
use Drupal\Core\Session\AccountProxy;
use Drupal\Core\TempStore\PrivateTempStoreFactory;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\RequestStack;
use Drupal\Core\DependencyInjection\ContainerInjectionInterface;

/**
 * Defines a controller for the content report.
 */
class ContentController implements ContainerInjectionInterface {

  /**
   * The request stack.
   *
   * @var \Symfony\Component\HttpFoundation\RequestStack
   */

  protected RequestStack $requestStack;

  /**
   * The pager manager service.
   *
   * @var \Drupal\Core\Pager\PagerManagerInterface
   */

  protected PagerManagerInterface $pagerManager;
  /**
   * The pager parameters service.
   *
   * @var \Drupal\Core\Pager\PagerParameters
   */

  protected PagerParameters $pagerParameters;
  /**
   * The date formatter.
   *
   * @var \Drupal\Core\Datetime\DateFormatterInterface
   */

  protected DateFormatterInterface $dateFormatter;
  /**
   * The current user.
   *
   * @var \Drupal\Core\Session\AccountProxy
   */

  protected AccountProxy $currentUser;
  /**
   * Temporary private data collection.
   *
   * @var \Drupal\Core\TempStore\PrivateTempStoreFactory
   */
  protected PrivateTempStoreFactory $tempStoreFactory;

  /**
   * The database connection.
   *
   * @var \Drupal\Core\Database\Connection
   */
  protected Connection $database;

  /**
   * The entity type manager service.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected EntityTypeManagerInterface $entityTypeManager;

  /**
   * The form builder service.
   *
   * @var \Drupal\Core\Form\FormBuilderInterface
   */
  protected FormBuilderInterface $formBuilder;

  /**
   * The module handler service.
   *
   * @var \Drupal\Core\Extension\ModuleHandlerInterface
   */
  protected ModuleHandlerInterface $moduleHandler;

  public function __construct(
    RequestStack $request_stack,
    EntityTypeManagerInterface $entityTypeManager,
    ModuleHandlerInterface $moduleHandler,
    PagerManagerInterface $pagerManager,
    PagerParameters $pagerParameters,
    DateFormatterInterface $dateFormatter,
    FormBuilderInterface $formBuilder,
    AccountProxy $currentUser,
    PrivateTempStoreFactory $tempStoreFactory,
    Connection $database,
  ) {
    $this->requestStack = $request_stack;
    $this->entityTypeManager = $entityTypeManager;
    $this->moduleHandler = $moduleHandler;
    $this->pagerManager = $pagerManager;
    $this->pagerParameters = $pagerParameters;
    $this->dateFormatter = $dateFormatter;
    $this->formBuilder = $formBuilder;
    $this->currentUser = $currentUser;
    $this->tempStoreFactory = $tempStoreFactory;
    $this->database = $database;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('request_stack'),
      $container->get('entity_type.manager'),
      $container->get('module_handler'),
      $container->get('pager.manager'),
      $container->get('pager.parameters'),
      $container->get('date.formatter'),
      $container->get('form_builder'),
      $container->get('current_user'),
      $container->get('tempstore.private'),
      $container->get('database'),
    );
  }

  /**
   * {@inheritdoc}
   */
  public function contentReport(Request $request) {
    $request = $this->requestStack->getCurrentRequest();

    // Get stored filters from tempstore.
    $tempstore = $this->tempStoreFactory->get('content_insights_report');
    $stored_filters = $tempstore->get('content_filters') ?: [];
    $limit = 50;

    // Get sort parameters.
    $sort = $stored_filters['sort'] ?? 'changed';
    // Allow sorting only on specific columns.
    $allowed_sorts = ['title', 'type', 'status', 'created', 'changed', 'revision_count', 'moderation_state'];
    if (!in_array($sort, $allowed_sorts)) {
      $sort = 'changed';
    }
    $order = $stored_filters['order'] ?? 'DESC';
    $allowed_status = ['0', '1'];

    // Get filter values from POST and validate dates.
    $cd_start_date = $this->validateDate($stored_filters['cd_start_date'] ?? NULL);
    $cd_end_date = $this->validateDate($stored_filters['cd_end_date'] ?? NULL);
    $ud_start_date = $this->validateDate($stored_filters['ud_start_date'] ?? NULL);
    $ud_end_date = $this->validateDate($stored_filters['ud_end_date'] ?? NULL);
    $rd_start_date = $this->validateDate($stored_filters['rd_start_date'] ?? NULL);
    $rd_end_date = $this->validateDate($stored_filters['rd_end_date'] ?? NULL);
    $content_type = $stored_filters['content_type'] ?? NULL;
    $title = $stored_filters['name'] ?? NULL;
    $status = $stored_filters['status'] ?? NULL;
    if (!in_array($status, $allowed_status)) {
      $status = NULL;
    }
    $moderation_state = $stored_filters['moderation_state'] ?? NULL;

    // Check if content moderation is enabled.
    $show_moderation = $this->moduleHandler->moduleExists('content_moderation');

    // Get moderation states if content moderation is enabled.
    $moderation_states = [];
    if ($show_moderation) {
      $workflow_storage = $this->entityTypeManager->getStorage('workflow');
      $workflows = $workflow_storage->loadMultiple();
      foreach ($workflows as $workflow) {
        if ($workflow->get('type') === 'content_moderation') {
          $type_plugin = $workflow->getTypePlugin();
          $states = $type_plugin->getStates();
          foreach ($states as $state_id => $state) {
            $moderation_states[$state_id] = $state->label();
          }
        }
      }
    }

    // Get all content types.
    $content_types = [];
    $node_types = $this->entityTypeManager->getStorage('node_type')->loadMultiple();
    foreach ($node_types as $type) {
      $content_types[$type->id()] = $type->label();
    }

    // Query to count nodes grouped by content type and moderation state.
    $connection = $this->database;
    $query = $connection->select('node_field_data', 'nfd');
    $query->fields('nfd', ['nid', 'title', 'type', 'status', 'created', 'changed']);

    if ($this->moduleHandler->moduleExists('content_moderation')) {
      // Subquery for revision count.
      $subquery_ms = $connection->select('content_moderation_state_field_data', 'cmsfd');
      $subquery_ms->fields('cmsfd', ['moderation_state', 'content_entity_id']);
      $subquery_ms->groupBy('cmsfd.moderation_state');
      $subquery_ms->groupBy('cmsfd.content_entity_id');

      // Join the subquery to the main query.
      $query->leftJoin($subquery_ms, 'rev_ms', 'nfd.nid = rev_ms.content_entity_id');
      $query->addExpression("COALESCE(rev_ms.moderation_state, 'N/A')", 'moderation_state');
    }

    // Subquery for revision count.
    $subquery = $connection->select('node_revision', 'nr');
    $subquery->fields('nr', ['nid']);
    $subquery->addExpression('COUNT(nr.vid)', 'revision_count');
    if ($rd_start_date) {
      $subquery->condition('nr.revision_timestamp', strtotime($rd_start_date), '>=');
    }
    if ($rd_end_date) {
      $subquery->condition('nr.revision_timestamp', strtotime($rd_end_date), '<=');
    }
    $subquery->groupBy('nr.nid');

    // Join the subquery to the main query.
    $query->leftJoin($subquery, 'rev', 'nfd.nid = rev.nid');
    $query->addExpression('COALESCE(rev.revision_count, 0)', 'revision_count');

    if ($content_type) {
      $query->condition('nfd.type', $content_type);
    }
    if ($title) {
      $query->condition('nfd.title', '%' . $title . '%', 'LIKE');
    }
    if ($status !== NULL && $status !== '') {
      $query->condition('nfd.status', (int) $status);
    }
    if ($this->moduleHandler->moduleExists('content_moderation') && $moderation_state) {
      $query->condition('moderation_state', $moderation_state);
    }
    if ($cd_start_date) {
      $query->condition('nfd.created', strtotime($cd_start_date), '>=');
    }
    if ($cd_end_date) {
      $query->condition('nfd.created', strtotime($cd_end_date), '<=');
    }
    if ($ud_start_date) {
      $query->condition('nfd.changed', strtotime($ud_start_date), '>=');
    }
    if ($ud_end_date) {
      $query->condition('nfd.changed', strtotime($ud_end_date), '<=');
    }
    if ($rd_start_date || $rd_end_date) {
      $query->condition('revision_count', 0, '>');
    }

    // Update sorting logic.
    switch ($sort) {
      case 'title':
        $query->orderBy('title', $order);
        break;

      case 'type':
        $query->orderBy('nfd.type', $order);
        break;

      case 'status':
        $query->orderBy('nfd.status', $order);
        break;

      case 'created':
        $query->orderBy('nfd.created', $order);
        break;

      case 'changed':
        $query->orderBy('nfd.changed', $order);
        break;

      case 'revision_count':
        $query->orderBy('rev.revision_count', $order);
        break;

      case 'moderation_state':
        $query->orderBy('moderation_state', $order);
        break;

      default:
        $query->orderBy('nfd.changed', 'DESC');
    }

    $pager = $query->extend(PagerSelectExtender::class);
    $pager->limit($limit);

    $results = $pager->execute()->fetchAll();
    $items = [];
    foreach ($results as $result) {
      // Load the node entity to get URLs.
      $node = $this->entityTypeManager->getStorage('node')->load($result->nid);

      $moderation_state_node = 'N/A';
      if ($node->hasField('moderation_state')) {
        if ($node->get('moderation_state')->value) {
          $moderation_state_node = $moderation_states[$node->get('moderation_state')->value];
        }
      }

      $items[] = [
        'title' => [
          'link' => $node->toUrl()->toString(),
          'label' => $node->getTitle(),
        ],
        'content_type' => $content_types[$node->bundle()],
        'status' => $node->isPublished() ? 'Published' : 'Unpublished',
        'moderation_state' => $moderation_state_node,
        'created' => $this->dateFormatter->format($node->getCreatedTime(), 'custom', 'd-m-Y'),
        'changed' => $this->dateFormatter->format($node->getChangedTime(), 'custom', 'd-m-Y'),
        'revision_count' => $result->revision_count,
        'edit_url' => $this->currentUser->hasPermission('edit any ' . $node->bundle() . ' content') ? $node->toUrl('edit-form')->toString() : NULL,
      ];
    }

    // Add the filter form to the render array.
    $form = $this->formBuilder->getForm('Drupal\content_insights_report\Form\ContentReportFilterForm');

    return [
      '#theme' => 'content_display',
      '#filter_form' => $form,
      '#data' => $items,
      '#pager' => [
        '#type' => 'pager',
      ],
      '#content_types' => $content_types,
      '#cd_start_date' => $cd_start_date,
      '#cd_end_date' => $cd_end_date,
      '#ud_start_date' => $ud_start_date,
      '#ud_end_date' => $ud_end_date,
      '#rd_start_date' => $rd_start_date,
      '#rd_end_date' => $rd_end_date,
      '#name' => $title,
      '#content_type' => $content_type,
      '#status' => $status,
      '#show_moderation' => $show_moderation,
      '#moderation_states' => $moderation_states,
      '#moderation_state' => $moderation_state,
      '#sort' => $sort,
      '#order' => $order,
      '#attached' => [
        'library' => [
          'content_insights_report/content_display',
        ],
      ],
    ];
  }

  /**
   * Validates a date string.
   *
   * @param string|null $date
   *   The date string to validate.
   *
   * @return string|null
   *   Returns the validated date string or NULL if invalid.
   */
  protected function validateDate($date) {
    if (empty($date)) {
      return NULL;
    }

    $datetime = \DateTime::createFromFormat('Y-m-d', $date);
    if ($datetime && $datetime->format('Y-m-d') === $date) {
      return $date;
    }

    return NULL;
  }

}
