<?php

namespace Drupal\content_insights_report\Controller;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Database\Connection;
use Drupal\Core\DependencyInjection\ContainerInjectionInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\Core\TempStore\PrivateTempStoreFactory;
use Drupal\Core\Url;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\RedirectResponse;
use Symfony\Component\HttpFoundation\Request;
use Drupal\Core\StringTranslation\TranslationManager;

/**
 * Defines a controller for the content_insights_report module.
 */
class ContentInsightsReportController implements ContainerInjectionInterface {

  /**
   * The config factory service.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected ConfigFactoryInterface $configFactory;

  /**
   * The entity type manager service.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected EntityTypeManagerInterface $entityTypeManager;

  /**
   * The module handler service.
   *
   * @var \Drupal\Core\Extension\ModuleHandlerInterface
   */
  protected $moduleHandler;

  /**
   * Temporary private data collection.
   *
   * @var \Drupal\Core\TempStore\PrivateTempStoreFactory
   */
  protected PrivateTempStoreFactory $tempStoreFactory;

  /**
   * The database connection.
   *
   * @var \Drupal\Core\Database\Connection
   */
  protected Connection $database;

  /**
   * The string translation service.
   *
   * @var \Drupal\Core\StringTranslation\TranslationManager
   */
  protected $stringTranslation;

  public function __construct(
    ConfigFactoryInterface $config_factory,
    EntityTypeManagerInterface $entity_type_manager,
    ModuleHandlerInterface $module_handler,
    PrivateTempStoreFactory $tempStoreFactory,
    Connection $database,
    TranslationManager $string_translation,
  ) {
    $this->configFactory = $config_factory;
    $this->entityTypeManager = $entity_type_manager;
    $this->moduleHandler = $module_handler;
    $this->tempStoreFactory = $tempStoreFactory;
    $this->database = $database;
    $this->stringTranslation = $string_translation;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('config.factory'),
      $container->get('entity_type.manager'),
      $container->get('module_handler'),
      $container->get('tempstore.private'),
      $container->get('database'),
      $container->get('string_translation'),
    );
  }

  /**
   * {@inheritdoc}
   */
  public function contentReport() {
    $config = $this->configFactory->get('content_insights_report.settings');
    $show_summary = $config->get('show_summary');
    $show_workflow_moderation_state_report = $config->get('show_workflow_moderation_state_report');
    $show_created_by_and_date = $config->get('show_created_by_and_date');
    $show_insights_report = $config->get('show_insights_report');
    $number_of_months = $config->get('number_of_months');
    $percentage_totals = $config->get('percentage_total');
    $sort_by_field = $config->get('sort_by_field');
    $sort_by_direction = $config->get('sort_by_direction');
    $show_print = $config->get('show_print');
    $show_empty_results = $config->get('show_empty_results');

    $report_data['content_type'] = [];

    switch ($percentage_totals) {
      case "published_nodes":
        // Sum of all published nodes.
        $percentage_totals_text = $this->stringTranslation->translate("Total percentage based on published nodes");
        break;

      case "all_nodes":
        // Sum of all nodes.
        $percentage_totals_text = $this->stringTranslation->translate("Total percentage based on all nodes");
        break;

      case "unpublished_nodes":
        // Sum of all published nodes.
        $percentage_totals_text = $this->stringTranslation->translate("Total percentage based on unpublished nodes");
        break;

    }

    // Node Counts.
    $report_data['node_counts'] = $this->getNodeCounts();

    // Insights Counts.
    $report_data['insights_counts'] = $this->getInsightsCounts();

    // Moderation State Report.
    if ($this->moduleHandler->moduleExists('content_moderation')) {
      $report_data['moderation_state'] = $this->getNodeSummaryPerModerationState();
    }

    // Sort result by.
    switch ($sort_by_field) {
      case "content_type":
        switch ($sort_by_direction) {
          case "asc":
            usort($report_data['node_counts']['content_type'], function ($a, $b) {
              return strcmp($a["content_type_name"], $b["content_type_name"]);
            });
            break;

          case "desc":
            usort($report_data['node_counts']['content_type'], function ($a, $b) {
              return strcmp($b["content_type_name"], $a["content_type_name"]);
            });
            break;

        }
        break;

      case "content_type_machine":
        switch ($sort_by_direction) {
          case "asc":
            usort($report_data['node_counts']['content_type'], function ($a, $b) {
              return strcmp($a["content_type_id"], $b["content_type_id"]);
            });
            break;

          case "desc":
            usort($report_data['node_counts']['content_type'], function ($a, $b) {
              return strcmp($b["content_type_id"], $a["content_type_id"]);
            });
            break;

        }
        break;

      case "total_percentage":
        switch ($sort_by_direction) {
          case "asc":
            usort($report_data['node_counts']['content_type'], function ($a, $b) {
              return ($a['published_percent'] - $b['published_percent']) ? ($a['published_percent'] - $b['published_percent']) / abs($a['published_percent'] - $b['published_percent']) : 0;
            });
            break;

          case "desc":
            usort($report_data['node_counts']['content_type'], function ($a, $b) {
              return ($b['published_percent'] - $a['published_percent']) ? ($b['published_percent'] - $a['published_percent']) / abs($b['published_percent'] - $a['published_percent']) : 0;
            });
            break;

        }
        break;

    }

    // Get current path for img.
    $modulePath = $this->moduleHandler->getModule('content_insights_report')->getPath();

    return [
      '#theme' => 'content_insights_report',
      '#show_summary' => $show_summary,
      '#show_insights_report' => $show_insights_report,
      '#show_workflow_moderation_state_report' => $show_workflow_moderation_state_report,
      '#show_created_by_and_date' => $show_created_by_and_date,
      '#show_empty_results' => $show_empty_results,
      '#months' => $number_of_months,
      '#show_print' => $show_print,
      '#image_path' => $modulePath . "/images/",
      '#report_data' => $report_data,
      '#percentage_totals' => $percentage_totals_text,
      '#attached' => [
        'library' => [
          'content_insights_report/content_insights_report',
        ],
      ],
    ];
  }

  /**
   * Count number of nodes based on status.
   *
   * @return int
   *   Count value of number of nodes.
   */
  private function getNodeCounts() {
    $config = $this->configFactory->get('content_insights_report.settings');
    $percentage_totals = $config->get('percentage_total');

    $connection = $this->database;

    // Create base array with content types and months.
    $types = $this->entityTypeManager->getStorage('node_type')->loadMultiple();
    $content_types_list = [];
    $content_types_list['total']['total_nodes'] = 0;
    $content_types_list['total']['published_nodes'] = 0;
    $content_types_list['total']['unpublished_nodes'] = 0;
    $content_types_list['total']['revision_nodes'] = 0;

    foreach ($types as $type) {
      $content_types_list['content_type'][$type->id()]['content_type_id'] = $type->id();
      $content_types_list['content_type'][$type->id()]['content_type_name'] = $type->label();
      $content_types_list['content_type'][$type->id()]['total_nodes'] = 0;
      $content_types_list['content_type'][$type->id()]['published_nodes'] = 0;
      $content_types_list['content_type'][$type->id()]['unpublished_nodes'] = 0;
      $content_types_list['content_type'][$type->id()]['revision_nodes'] = 0;
      $content_types_list['content_type'][$type->id()]['published_percent'] = 0;
    }

    // Get published and unpublished nodes counts.
    $subquery_created = $connection->select('node_field_data', 'nfd');
    $subquery_created->fields('nfd', ['type', 'status']);
    $subquery_created->addExpression('COUNT(DISTINCT nfd.nid)', 'count');
    $subquery_created->groupBy('type');
    $subquery_created->groupBy('status');
    $results_created = $subquery_created->execute()->fetchAll();

    // Get revision counts.
    $subquery_revision = $connection->select('node_revision', 'rev');
    $subquery_revision->leftJoin('node_field_data', 'nfd', 'rev.nid = nfd.nid');
    $subquery_revision->fields('nfd', ['type']);
    $subquery_revision->addExpression('COUNT(DISTINCT rev.vid)', 'count');
    $subquery_revision->groupBy('type');
    $results_revision = $subquery_revision->execute()->fetchAll();

    // Combine all results into the base array.
    foreach ($results_created as $result) {
      if (isset($content_types_list['content_type'][$result->type])) {
        if ($result->status == 1) {
          $content_types_list['content_type'][$result->type]['published_nodes'] = $result->count;
          $content_types_list['total']['published_nodes'] += $result->count;
        }
        else {
          $content_types_list['content_type'][$result->type]['unpublished_nodes'] = $result->count;
          $content_types_list['total']['unpublished_nodes'] += $result->count;
        }
        $content_types_list['content_type'][$result->type]['total_nodes'] += $result->count;
        $content_types_list['total']['total_nodes'] += $result->count;
      }
    }
    foreach ($results_revision as $result) {
      if (isset($content_types_list['content_type'][$result->type])) {
        $content_types_list['content_type'][$result->type]['revision_nodes'] = $result->count;
        $content_types_list['total']['revision_nodes'] += $result->count;
      }
    }

    foreach ($content_types_list['content_type'] as $key => $result) {
      switch ($percentage_totals) {
        case "all_nodes":
          // Sum of all nodes.
          $content_types_list['content_type'][$key]['published_percent'] = $content_types_list['total']['total_nodes'] ? (($result['published_nodes'] + $result['unpublished_nodes']) / $content_types_list['total']['total_nodes']) * 100 : 0;
          break;

        case "published_nodes":
          // Sum of all published nodes.
          $content_types_list['content_type'][$key]['published_percent'] = $content_types_list['total']['total_nodes'] ? ($result['published_nodes'] / $content_types_list['total']['total_nodes']) * 100 : 0;
          break;

        case "unpublished_nodes":
          // Sum of all published nodes.
          $content_types_list['content_type'][$key]['published_percent'] = $content_types_list['total']['total_nodes'] ? ($result['unpublished_nodes'] / $content_types_list['total']['total_nodes']) * 100 : 0;
          break;
      }
    }

    return $content_types_list;
  }

  /**
   * Insights Counts values.
   *
   * @return array
   *   An array of content type statistics.
   */
  private function getInsightsCounts() {

    $config = $this->configFactory->get('content_insights_report.settings');
    $number_of_months = $config->get('number_of_months');

    $connection = $this->database;

    // Get last 6 months date range.
    $end_date = new \DateTime();
    $start_date = (new \DateTime())->modify('-' . $number_of_months . ' months');

    // Create base array with content types and months.
    $types = $this->entityTypeManager->getStorage('node_type')->loadMultiple();
    $content_types_month_list = [];

    foreach ($types as $type) {
      $date = new \DateTime();
      for ($i = 0; $i < $number_of_months; $i++) {
        $year_month = $date->format('Ym');

        $content_types_month_list['headings'][$year_month] = [
          'month' => $date->format('F Y'),
          'start_of_month' => $date->format('Y-m-01'),
          'end_of_month' => $date->format('Y-m-t'),
        ];
        $content_types_month_list[$type->id()]['created'][$year_month] = 0;
        $content_types_month_list[$type->id()]['updated'][$year_month] = 0;
        $content_types_month_list[$type->id()]['revision'][$year_month] = 0;

        $date->modify('-1 month');
      }
    }

    // Get created nodes counts.
    $subquery_created = $connection->select('node_field_data', 'nfd_created');
    $subquery_created->fields('nfd_created', ['type']);
    $subquery_created->addExpression("FROM_UNIXTIME(nfd_created.created, '%Y%m')", 'year_month');
    $subquery_created->addExpression('COUNT(DISTINCT nfd_created.nid)', 'created_nodes');
    $subquery_created->condition('nfd_created.created', strtotime($start_date->format('Y-m-01')), '>=');
    $subquery_created->condition('nfd_created.created', strtotime($end_date->format('Y-m-t')), '<=');
    $subquery_created->groupBy('year_month');
    $subquery_created->groupBy('type');
    $results_created = $subquery_created->execute()->fetchAll();

    // Get updated nodes counts.
    $subquery_updated = $connection->select('node_field_data', 'nfd_updated');
    $subquery_updated->fields('nfd_updated', ['type']);
    $subquery_updated->addExpression("FROM_UNIXTIME(nfd_updated.changed, '%Y%m')", 'year_month');
    $subquery_updated->addExpression('COUNT(DISTINCT nfd_updated.nid)', 'updated_nodes');
    $subquery_updated->condition('nfd_updated.changed', strtotime($start_date->format('Y-m-01')), '>=');
    $subquery_updated->condition('nfd_updated.changed', strtotime($end_date->format('Y-m-t')), '<=');
    $subquery_updated->groupBy('year_month');
    $subquery_updated->groupBy('type');
    $results_updated = $subquery_updated->execute()->fetchAll();

    // Get revision counts.
    $subquery_revision = $connection->select('node_revision', 'rev');
    $subquery_revision->leftJoin('node_field_data', 'nfd', 'rev.nid = nfd.nid');
    $subquery_revision->fields('nfd', ['type']);
    $subquery_revision->addExpression("FROM_UNIXTIME(rev.revision_timestamp, '%Y%m')", 'year_month');
    $subquery_revision->addExpression('COUNT(DISTINCT rev.vid)', 'revision_nodes');
    $subquery_revision->condition('rev.revision_timestamp', strtotime($start_date->format('Y-m-01')), '>=');
    $subquery_revision->condition('rev.revision_timestamp', strtotime($end_date->format('Y-m-t')), '<=');
    $subquery_revision->groupBy('year_month');
    $subquery_revision->groupBy('type');
    $results_revision = $subquery_revision->execute()->fetchAll();

    // Combine all results into the base array.
    foreach ($results_created as $result) {
      if (isset($content_types_month_list[$result->type])) {
        $content_types_month_list[$result->type]['created'][$result->year_month] = $result->created_nodes;
      }
    }

    foreach ($results_updated as $result) {
      if (isset($content_types_month_list[$result->type])) {
        $content_types_month_list[$result->type]['updated'][$result->year_month] = $result->updated_nodes;
      }
    }

    foreach ($results_revision as $result) {
      if (isset($content_types_month_list[$result->type])) {
        $content_types_month_list[$result->type]['revision'][$result->year_month] = $result->revision_nodes;
      }
    }

    return $content_types_month_list;
  }

  /**
   * Generates based on content type, status, and moderation state.
   *
   * @return array
   *   An array of counts grouped by content type, status, and moderation state.
   */
  private function getNodeSummaryPerModerationState() {

    $moderation_state = [];
    $connection = $this->database;

    $types = $this->entityTypeManager->getStorage('node_type')->loadMultiple();
    foreach ($types as $type) {

      $workflows = $this->entityTypeManager->getStorage('workflow')->loadMultiple();
      // Loop through each workflow and get its states.
      foreach ($workflows as $workflow) {
        // Get the states defined for this workflow.
        $workflow_states = $workflow->getTypePlugin()->getStates();
        foreach ($workflow_states as $state) {
          // Published Moderation state.
          $moderation_state[$type->id()][$workflow->id()]['name'] = $workflow->label();
          $moderation_state[$type->id()][$workflow->id()]['total_count'] = 0;
          $moderation_state[$type->id()][$workflow->id()]['status']['1']['name'] = "Published";
          $moderation_state[$type->id()][$workflow->id()]['status']['1']['stage'][$state->id()]['name'] = $state->label();
          $moderation_state[$type->id()][$workflow->id()]['status']['1']['stage'][$state->id()]['count'] = 0;
          // Unpublished Moderation state.
          $moderation_state[$type->id()][$workflow->id()]['status']['0']['name'] = "Unpublished";
          $moderation_state[$type->id()][$workflow->id()]['status']['0']['stage'][$state->id()]['name'] = $state->label();
          $moderation_state[$type->id()][$workflow->id()]['status']['0']['stage'][$state->id()]['count'] = 0;
        }
      }
    }

    $query = $connection->select('node_field_data', 'nfd');
    $query->fields('nfd', [
      'type',
      'status',
    ]);

    if ($this->moduleHandler->moduleExists('content_moderation')) {
      // Subquery for revision count.
      $subquery_ms = $connection->select('content_moderation_state_field_data', 'cmsfd');
      $subquery_ms->fields('cmsfd', [
        'moderation_state',
        'content_entity_id',
        'workflow',
      ]);
      $subquery_ms->groupBy('cmsfd.moderation_state');
      $subquery_ms->groupBy('cmsfd.content_entity_id');
      $subquery_ms->groupBy('cmsfd.workflow');

      // Join the subquery to the main query.
      $query->leftJoin($subquery_ms, 'rev_ms', 'nfd.nid = rev_ms.content_entity_id');
      $query->addExpression("COALESCE(rev_ms.moderation_state, 'N/A')", 'moderation_state');
      $query->fields('rev_ms', ['workflow']);
    }
    $query->addExpression('COUNT(nfd.nid)', 'node_count');
    $query->isNotNull('rev_ms.workflow');
    $query->groupBy('nfd.type');
    $query->groupBy('nfd.status');
    $query->groupBy('rev_ms.workflow');
    $query->groupBy('rev_ms.moderation_state');

    $result = $query->execute();

    foreach ($result as $record) {
      $moderation_state[$record->type][$record->workflow]['status'][$record->status]['stage'][$record->moderation_state ?: 'no_moderation_state']['count'] = $record->node_count;
      $moderation_state[$record->type][$record->workflow]['total_count'] += $record->node_count;
    }

    return $moderation_state;
  }

  /**
   * Updates filters based on AJAX request.
   *
   * @param \Symfony\Component\HttpFoundation\Request $request
   *   The current request object.
   *
   * @return \Symfony\Component\HttpFoundation\JsonResponse
   *   Return string.
   */
  public function updateFilters(Request $request) {
    $filters = [];

    // Get parameters from the request.
    $params = [
      'sort',
      'order',
      'cd_start_date',
      'cd_end_date',
      'ud_start_date',
      'ud_end_date',
      'rd_start_date',
      'rd_end_date',
      'name',
      'content_type',
      'status',
      'moderation_state',
    ];

    foreach ($params as $param) {
      $value = $request->query->get($param);
      $filters[$param] = $value;
    }

    // Store filters in tempstore.
    $tempstore = $this->tempStoreFactory->get('content_insights_report');
    $tempstore->set('content_filters', $filters);

    // Redirect to content display page.
    return new RedirectResponse(Url::fromRoute('content_insights_report.content_display')->toString());
  }

}
