<?php

namespace Drupal\content_insights_report\Form;

use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\TempStore\PrivateTempStoreFactory;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Form for Content Insights Report Content report.
 */
class ContentReportFilterForm extends FormBase {

  /**
   * The private tempstore factory.
   *
   * @var \Drupal\Core\TempStore\PrivateTempStoreFactory
   */
  protected $tempStoreFactory;

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected EntityTypeManagerInterface $entityTypeManager;

  /**
   * The module handler service.
   *
   * @var \Drupal\Core\Extension\ModuleHandlerInterface
   */
  protected $moduleHandler;

  public function __construct(
    PrivateTempStoreFactory $tempStoreFactory,
    EntityTypeManagerInterface $entity_type_manager,
    ModuleHandlerInterface $module_handler,
  ) {
    $this->tempStoreFactory = $tempStoreFactory;
    $this->entityTypeManager = $entity_type_manager;
    $this->moduleHandler = $module_handler;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('tempstore.private'),
      $container->get('entity_type.manager'),
      $container->get('module_handler'),
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'content_report_filter_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $form['#method'] = 'POST';

    // Get stored filters from tempstore.
    $tempstore = $this->tempStoreFactory->get('content_insights_report');
    $stored_filters = $tempstore->get('content_filters') ?: [];

    // Add hidden sort fields.
    $form['sort'] = [
      '#type' => 'hidden',
      '#default_value' => $stored_filters['sort'] ?? 'changed',
    ];

    $form['order'] = [
      '#type' => 'hidden',
      '#default_value' => $stored_filters['order'] ?? 'DESC',
    ];

    $form['filters'] = [
      '#type' => 'fieldset',
      '#attributes' => ['class' => ['content-filters']],
    ];

    $date_fields = [
      'cd' => 'Creation Date',
      'ud' => 'Update Date',
      'rd' => 'Revision Date',
    ];

    foreach ($date_fields as $key => $label) {
      $form['filters'][$key] = [
        '#type' => 'fieldset',
        '#title' => $this->t('@label', ['@label' => $label]),
        '#collapsible' => TRUE,
        '#collapsed' => FALSE,
      ];
      $form['filters'][$key][$key . '_start_date'] = [
        '#type' => 'date',
        '#title' => $this->t('Start Date'),
        '#default_value' => $stored_filters[$key . '_start_date'] ?? NULL,
        '#attributes' => [
          'max' => date('Y-m-d'),
        ],
      ];

      $form['filters'][$key][$key . '_end_date'] = [
        '#type' => 'date',
        '#title' => $this->t('End Date'),
        '#default_value' => $stored_filters[$key . '_end_date'] ?? NULL,
        '#attributes' => [
          'max' => date('Y-m-d'),
        ],
      ];
    }

    $form['filters']['other_filters'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('Other Filters'),
      '#collapsible' => TRUE,
      '#collapsed' => FALSE,
    ];

    $form['filters']['other_filters']['name'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Title'),
      '#default_value' => $stored_filters['name'] ?? NULL,
    ];

    // Get content types with their labels.
    $content_types = [];
    $types = $this->entityTypeManager->getStorage('node_type')->loadMultiple();
    foreach ($types as $type) {
      $content_types[$type->id()] = $type->label();
    }

    $form['filters']['other_filters']['content_type'] = [
      '#type' => 'select',
      '#title' => $this->t('Content Type'),
      '#options' => ['' => $this->t('- Any -')] + $content_types,
      '#default_value' => $stored_filters['content_type'] ?? NULL,
      '#attributes' => [
        'style' => 'min-width: 170px;',
      ],
    ];

    $form['filters']['other_filters']['status'] = [
      '#type' => 'select',
      '#title' => $this->t('Status'),
      '#options' => [
        '' => $this->t('- Any -'),
        '1' => $this->t('Published'),
        '0' => $this->t('Unpublished'),
      ],
      '#default_value' => $stored_filters['status'] ?? NULL,
      '#attributes' => [
        'style' => 'min-width: 170px;',
      ],
    ];

    // Add moderation state if content moderation is enabled.
    if ($this->moduleHandler->moduleExists('content_moderation')) {
      $workflow_storage = $this->entityTypeManager->getStorage('workflow');
      $workflows = $workflow_storage->loadMultiple();
      $moderation_states = [];

      foreach ($workflows as $workflow) {
        if ($workflow->get('type') === 'content_moderation') {
          $type_plugin = $workflow->getTypePlugin();
          $states = $type_plugin->getStates();
          foreach ($states as $state_id => $state) {
            $moderation_states[$state_id] = $state->label();
          }
        }
      }

      if (!empty($moderation_states)) {
        $form['filters']['other_filters']['moderation_state'] = [
          '#type' => 'select',
          '#title' => $this->t('Moderation State'),
          '#options' => ['' => $this->t('- Any -')] + $moderation_states,
          '#default_value' => $stored_filters['moderation_state'] ?? NULL,
          '#attributes' => [
            'style' => 'min-width: 170px;',
          ],
        ];
      }
    }

    $form['filters']['other_filters']['submit'] = [
      '#type' => 'submit',
      '#value' => $this->t('Filter'),
    ];

    $form['filters']['other_filters']['reset'] = [
      '#type' => 'submit',
      '#value' => $this->t('Reset'),
      '#submit' => ['::resetForm'],
    ];

    // Add form validation.
    $form['#validate'][] = '::validateDateFields';

    return $form;
  }

  /**
   * Validates the date fields to ensure they are not in the future.
   */
  public function validateDateFields(array &$form, FormStateInterface $form_state) {
    $date_fields = ['cd', 'ud', 'rd'];
    $today = new \DateTime('today');

    foreach ($date_fields as $field) {
      $start_date = $form_state->getValue($field . '_start_date');
      $end_date = $form_state->getValue($field . '_end_date');

      if (!empty($start_date)) {
        $start = new \DateTime($start_date);
        if ($start > $today) {
          $form_state->setErrorByName(
            $field . '_start_date',
            $this->t('Start date cannot be in the future.')
          );
        }
      }

      if (!empty($end_date)) {
        $end = new \DateTime($end_date);
        if ($end > $today) {
          $form_state->setErrorByName(
            $field . '_end_date',
            $this->t('End date cannot be in the future.')
          );
        }
      }
    }
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $filters = [];

    // Get sort and order values.
    $sort = $form_state->getValue('sort');
    $order = $form_state->getValue('order');
    if (!empty($sort)) {
      $filters['sort'] = $sort;
    }
    if (!empty($order)) {
      $filters['order'] = $order;
    }

    // Get all the date filter values.
    $date_fields = ['cd', 'ud', 'rd'];
    foreach ($date_fields as $field) {
      $start_date = $form_state->getValue($field . '_start_date');
      $end_date = $form_state->getValue($field . '_end_date');
      if (!empty($start_date)) {
        $filters[$field . '_start_date'] = $start_date;
      }
      if (!empty($end_date)) {
        $filters[$field . '_end_date'] = $end_date;
      }
    }

    // Get other filter values.
    $other_filters = [
      'name',
      'content_type',
      'status',
      'moderation_state',
    ];

    foreach ($other_filters as $filter) {
      $value = $form_state->getValue($filter);
      if (!empty($value)) {
        $filters[$filter] = $value;
      }
    }

    // Store filters in session.
    $tempstore = $this->tempStoreFactory->get('content_insights_report');
    $tempstore->set('content_filters', $filters);

    // Redirect to the content display page.
    $form_state->setRedirect('content_insights_report.content_display');
  }

  /**
   * {@inheritdoc}
   */
  public function resetForm(array &$form, FormStateInterface $form_state) {
    // Clear the filters from session.
    $tempstore = $this->tempStoreFactory->get('content_insights_report');
    $tempstore->delete('content_filters');

    $form_state->setRedirect('content_insights_report.content_display');
  }

}
